/*
 * production.cpp
 *
 *  Created on: Aug 20, 2017
 *      Author: wn
 */
#include <stdbool.h>

#include "defines.h"

#define MQTT_MAX_PACKET_SIZE 256


#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>

#define ESP8266
#include <PubSubClient.h>

#include "configuration.h"



WiFiClient espClient;
PubSubClient client(espClient);

const char watchdogTopic[] = "IoT/Watchdog";
bool watchdogEnabled = false;


void setup_wifi() {
  delay(10);
  WiFi.mode(WIFI_STA);

  // We start by connecting to a WiFi network
#ifdef DEBUG
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(configBlock.wifiSsid);
#endif

  WiFi.begin(configBlock.wifiSsid, configBlock.wifiKey);

  while (WiFi.status() != WL_CONNECTED) {
    delay(50);
#ifdef DEBUG
    Serial.print(".");
#endif
  }

#ifdef DEBUG
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
#endif
}




void callback(char* topic, byte* payload, unsigned int length) {
  Serial.println("callback");
  const uint16_t BUFSIZE = 256;
  if ((length + 1) >= BUFSIZE) {  // 1 for terminating NUL
#ifdef DEBUG
	  Serial.print(length);
	  Serial.println(", received message too long, ignore it");

#endif
  } else {
    char buffer[BUFSIZE];
    memcpy(buffer, payload, length);
    *(buffer + length) = 0;
#ifdef DEBUG
    Serial.print("Received message: ");
    Serial.print(length);
    Serial.print(" ");
    Serial.print(topic);
    Serial.print(" ");
    Serial.println(buffer);
#endif

    if (! strcmp(topic, configBlock.mqttTopic)) {
    	if (! strcmp(buffer, "ON")) {
    		digitalWrite(LIGHT_PIN, HIGH);
    	} else {
    		digitalWrite(LIGHT_PIN, LOW);
    	}
    }

    if (! strcmp(topic, watchdogTopic)) {
    	Serial.println("Feed watchdog");
    	ESP.wdtFeed();
    }
  }
}



void setupProduction() {
  setup_wifi();
  client.setServer(configBlock.mqttBroker, 1883);
  client.setCallback(callback);

  pinMode(LIGHT_PIN, OUTPUT);

}


void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
#ifdef DEBUG
    Serial.print("Attempting MQTT connection...");
#endif
    if (client.connect(configBlock.mqttClientId)) {
#ifdef DEBUG
      Serial.println("connected");
#endif
      client.subscribe(configBlock.mqttTopic);
      client.subscribe(watchdogTopic);

      client.publish(configBlock.mqttDebugTopic, "hello world");
      client.publish(configBlock.mqttDebugTopic, WiFi.localIP().toString().c_str());

      if (! watchdogEnabled) {
    	  watchdogEnabled = true;
    	  ESP.wdtEnable(25000);
      }
    } else {
#ifdef DEBUG
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
#endif
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}


void mqtt_connect() {
  if (!client.connected()) {
    reconnect();
  }
  client.loop();
}

void loopProduction() {
  mqtt_connect();
}




